#
# $Copyright (c) 2022 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other 
# names may be trademarks of their respective owners.
# 
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $
# $Revision: 1.4.5.44 $
# $Source: /project/arrayrep/src/srdfsnap/unix/SRDFSnap.pm,v $
#

package SRDFSnap;

BEGIN { push @INC, '../SRDF'; }
use SRDFAgent;
use ArrayRep;
@ISA = ("ArrayRep");
use POSIX qw(uname);

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default %vmattrib);

@EXPORT = qw(
);
$VERSION = '1.00';


# All ha-cmds have been defined as constants in ag_i18n_inc.pm. We can
# simply use the defined constants for invoking the ha-cmds.
use ag_i18n_inc;

my ($OS) = POSIX::uname();
# Fix for leadville devices
$ENV{"SYMCLI_FULL_PDEVNAME"} = 1;

my $vcs_home=$VCSHOME;

my $VXDG = "/usr/sbin/vxdg";
my $VXDISK = "/usr/sbin/vxdisk";
my $VXASSIST;

my $LQUERYVG = "/usr/sbin/lqueryvg";
my $GETLVODM = "/usr/sbin/getlvodm";
my $RECREATEVG = "/usr/sbin/recreatevg";
my $RECREATEVGTIMER = 15;   # If recreatevg fails, how many times do we retry 
my $RECREATEVGINTVL = 5;    # If recreatevg fails, interval to wait before retrying
my $CHDEV = "/usr/sbin/chdev";
my $ODMGET = "/usr/bin/odmget";
my $EXPORTVG = "/usr/sbin/exportvg";

my $HACONF = "$VCSHOME/bin/haconf";
my $VXDCTL = "/usr/sbin/vxdctl";
my $VXDISKCONFIG = "/usr/sbin/vxdiskconfig";
my $VXCLUSTADM = "/etc/vx/bin/vxclustadm";
my $VXDMPADM = "/opt/VRTS/bin/vxdmpadm";
my $DEVNULL = "/dev/null";
my $VXPATH = "/dev/vx/rdmp";

#Scan devices
my $DEVFSADM = "/usr/sbin/devfsadm";
my $IOSCAN = "/usr/sbin/ioscan";
my $CFGMGR = "/usr/sbin/cfgmgr";

my ($SYMRDF, $SYMDG, $SYMCG, $SYMMIR, $SYMCLONE, $SYMCLI, $SYMSNAP, $SYMINQ);

# Name of the resource type for DiskGroup.
# This would be CVMVolDg in a RAC environment.
my $DGTYPE = "DiskGroup"; 

# Name of the attribute that stores the DiskGroup name.
# This would be CVMDiskGroup in a RAC environment.
my $DGATTRNAME = "DiskGroup";

my $osType = $^O;

%default = (
	resname => '',
	target => '',
	sotype => '',
	basedir => '',
	group => '',
	instant => '',
	dgname 	=> '',  # A list of dgs
	RAC     => '',	# = 1 if this FD is being run in a RAC env
			# = 0, otherwise

	dcv_mode => '',
	iscompositegroup => 0,
	grptype =>'',

	#P-i-T Copy / Snapshot Mode 
	copymode => '',
	tgtflag => '',

	# Type of volume manager managing the replicated disks.
	# Currently the following vmtypes are supported: LVMVG/
	# DiskGroup/CVMVolDg/LPAR
	vmtype => '',

	# Set to 1 if the service grp containing the resource
	# is a parallel service grp; 0 otherwise
	rac => '',

	# Stores the value of OnlineTimeout - used for optimization 
	# only so that we dont run the HATYPE command to get the 
	# online timeout several times in the code
	online_tmo => '',

	# resource type name. In this case it will be SRDFSnap
	resource_type => '',

	#Save Pool name only if copymode is set to snap
	savepoolname => '',
	symcli_ver => '0'
);

#
# Map of VM technology to name of attribute that contains the
# diskgroup/volumegroup name
#
%vmattrib = (
	LVMVG => 'VolumeGroup',
	LPAR => 'LPAR',
	DiskGroup => 'DiskGroup',
	CVMVolDg => 'CVMDiskGroup',
    VMDg => 'DiskGroupName'
);

sub new {
	my ($proto, $resname, $target, $mountSO, $useSO, $requireSO, $savepoolname, $CopyMode, $UseTgt) = @_;
	my ($group, $basedir);
	my $class = ref($proto) || $proto;
	my @dgname;
	my @accepted_val_CopyMode=("mirror", "snap", "clone");

	my $self = {%default};
	$self->{'resource_type'} = "SRDFSnap";
	$self->{'resname'} = $resname;
	$self->{'target'} = $target;
	$self->{'symcli_ver'} = 0;

	if ($mountSO) {
		if ($requireSO) {
			$self->{'sotype'} = "gold-only";
		} else {
			$self->{'sotype'} = 'gold';
		}
	} elsif ($useSO) {
		if ($requireSO) {
			$self->{'sotype'} = "silver-only";
		} else {
			$self->{'sotype'} = 'silver';
		}
	} else {
		$self->{'sotype'} = "bronze";
	}

	$self->{'copymode'} = lc($CopyMode);
	if ( !grep(/^$self->{'copymode'}$/, @accepted_val_CopyMode)){
		VCSAG_LOG_MSG("E", "Unsupported type of snapshot technology $self->{'copymode'}", 142, "$self->{'copymode'}");
	return undef;
	}
	$self->{'savepoolname'} = $savepoolname;
	$self->{'instant'} = 0;
	$self->{'rac'} = 0;

	##################### BEGIN CHANGES FOR RAC FD ##################
	my ($sg) = split('\n', `$ag_i18n_inc::HARES -value $target Group`);
	my ($pll) = split('\n', `$ag_i18n_inc::HAGRP -value $sg Parallel`);
	if ($pll == 1) {
		$self->{'rac'} = 1;
	}
	###################### END CHANGES FOR RAC FD ###################

	# get DG name
	my ($typ) = split('\n', `$ag_i18n_inc::HARES -value $target Type`);
	my @dgname;
	my $index = 0;
	my @tmp = "";
	if ($typ eq "SRDF") {
		my (@depfields) = split('\n', `$ag_i18n_inc::HARES -dep $target`);
        my $dgattrib = "";
		foreach my $depline (@depfields) {
			my (@deppair) = split('\s+', $depline);
			if ($deppair[2] eq $target) {
				@tmp = split('\n', `$ag_i18n_inc::HARES -value $deppair[1] Type`);
                if (0 == length $self->{'vmtype'}) {
                    $self->{'vmtype'} = $tmp[0];
                    if (!exists($vmattrib{$self->{'vmtype'}})) {
                        VCSAG_LOG_MSG("E", "Unsupported type of volume manager $tmp[0]", 123, "$tmp[0]");
                        return undef;
                    }
                    $dgattrib = $vmattrib{$self->{'vmtype'}};
                } else {
                    if ($self->{'vmtype'} ne $tmp[0]) {
                        VCSAG_LOG_MSG("E", "Mixed Volume Management types are not supported; Exiting", 131);
                        return undef;
                    }
                }
                @tmp = split('\n', `$ag_i18n_inc::HARES -value $deppair[1] $dgattrib`);
				$dgname[$index] = $tmp[0];
				$index = $index + 1;
			}
		}
		$self->{'dgname'} = join(' ', "@dgname");
		@tmp = split('\n', `$ag_i18n_inc::HARES -value $target GrpName`);
		$self->{'group'} = $tmp[0];
		@tmp = split('\n', `$ag_i18n_inc::HARES -value $target SymHome`);
		$self->{'basedir'} = $tmp[0];
	} else {
		my ($sg) = split('\n', `$ag_i18n_inc::HARES -value $target Group`);
		my (@rdfres) = split('\n', `$ag_i18n_inc::HARES -list Type=SRDF Group=$sg`);
		my (@rdfresn) = split('\s+', $rdfres[0]);
		($self->{'basedir'}) = split('\n', `$ag_i18n_inc::HARES -value $rdfresn[0] SymHome`);
        $self->{'vmtype'} = $typ;
		@tmp = split('\n', `$ag_i18n_inc::HARES -value $target $vmattrib{$self->{'vmtype'}}`);
		$dgname[$index] = $tmp[0];
		$self->{'dgname'} = join(' ',@dgname);
		$self->{'group'} = $tmp[0];
		$self->{'instant'} = 1;
	}

    if (($self->{'vmtype'} eq "LVMVG") && ($OS !~ /AIX/i)) {
		VCSAG_LOG_MSG("E", "LVM is supported only on AIX; Exiting", 135);
		return undef;
	}

	if ( $osType =~ /^MSWin/i)
	{
		require Win32;
		Win32->import;
    
		my $SYMHOME = $self->{'basedir'};
		$SYMHOME = Win32::GetShortPathName($SYMHOME);
		$self->{'basedir'} = $SYMHOME;
		$SYMRDF = "\"$SYMHOME\\symrdf\"";
		$SYMDG = "\"$SYMHOME\\symdg\"";
		$SYMCG = "\"$SYMHOME\\symcg\"";
		$SYMMIR   = "\"$SYMHOME\\symmir\"";
		$SYMSNAP  = "\"$SYMHOME\\symsnap\"";
		$SYMCLONE = "\"$SYMHOME\\symclone\"";
		$SYMCLI = "\"$SYMHOME\\symcli\"";
		$SYMINQ = "\"$SYMHOME\\syminq\"";
		$self->{'rac'} = 0;
        
		my $VMPATH = $ENV{"VMPATH"};
		$VMPATH = Win32::GetShortPathName($VMPATH);
		$VXDG = $VMPATH . "vxdg";
		$VXDISK = $VMPATH . "vxdisk";
   		$VXASSIST = $VMPATH . "vxassist";
		$VXDG = "\"$VXDG\"";
		$VXDISK = "\"$VXDISK\"";
   		$VXASSIST = "\"$VXASSIST\"";

        $DEVNULL = "nul";

    } else {
        $SYMRDF = "$self->{'basedir'}/bin/symrdf";
        $SYMMIR = "$self->{'basedir'}/bin/symmir";
        $SYMINQ = "$self->{'basedir'}/bin/syminq";
        $SYMDG = "$self->{'basedir'}/bin/symdg";
        $SYMCG = "$self->{'basedir'}/bin/symcg";
        $SYMSNAP = "$self->{'basedir'}/bin/symsnap";
		$SYMCLONE = "$self->{'basedir'}/bin/symclone";
		$SYMCLI = "$self->{'basedir'}/bin/symcli";
    }
	
	if ($UseTgt == 1) {
		$self->{'tgtflag'} = "-tgt";
	}   		

	#
	# strip leading spaces if any
	#
	$self->{'dgname'} =~ s/^[\s]+//;
	bless($self, $class);
    return $self;
}

sub DESTROY {}


sub check_empty {
	my $ResName = shift;
	my @attributes = @_;
	my $total_attrib = $#attributes;
	my $index = 0;
	while ($index < $total_attrib) {
		my $attribute = $attributes[$index];
		$index ++;
		if (!$attributes[$index]) {
			VCSAG_LOG_MSG("E", "Attribute $attribute not defined for $ResName.", 148, $attribute, $ResName);
			return 1;
		} else {
			$index ++;
		}
	}
	return 0;
}
	


sub get_mode {
	my ($self, $type) = @_;
	my $ret = 0;
	my @tmparr = ();
	my $func = "get_mode";
	return 0 if (exists ($vmattrib{$type}));
	my $cmd = "$SYMRDF $self->{'grptype'} $self->{'group'} query";
    VCSAG_LOGDBG_MSG(1, "get_mode: Issuing cmd: [$cmd]");
	my $output = `$cmd`;
	if (!$output) {
		$ret = "$?";
		VCSAG_LOG_MSG("E", "Command [$cmd] failed with error code: $ret", 101, "$cmd", "$ret");
		return 1;
	}

	if (SRDFAgent::run_cmd("$SYMCLI -v", \@tmparr, 1, $func)) {
                return 0;
        }

        $self->extract_CLI_version(\@tmparr);

	my @query = split('\n', $output);
        my $modepat= qr/[SAD][X\.]/;
        if($self->{'symcli_ver'}<9.0) {
            $modepat= qr/[SAC]\./;
        }
        foreach my $q (@query) {
                if (!($q =~ $modepat)) {
                        next;
                }
		my @modeinfo = split('\s+', $q);
		if ($modeinfo[10] =~ /S/) {
			$self->{'dcv_mode'} = "Synchronous";
			last;
		}elsif ($modeinfo[10] =~ /A/) {
			$self->{'dcv_mode'} = "Asynchronous";
			last;
		}
	}
	if ($self->{'dcv_mode'} eq "") {
		VCSAG_LOG_MSG("E", "Error getting the mode of replication", 102);
		$ret = 1;
    }
	return $ret;
}

sub is_SYMCLI_version_6 {
    my ($self) = shift;
	my @tmparr = ();
	
	if (!run_cmd_with_output("$SYMCLI", \@tmparr)) {
		return 2; 
	}
	if ( grep (/V6\./, @tmparr)) {
		return 1;
	}
	return 0;  # 7 and above
}


sub take_snapshot {
	my $self = shift;
	my $is_rac_cluster = $self->{'rac'};
	my $is_master = 0;
	my $ret = 0;
	my @cmds = ();

	if ($is_rac_cluster == 1) {
		# This is a RAC environment i.e. the group for which a 
		# FireDrill is being run is a parallel group.
		$is_master = ArrayRep::is_cvm_master();

		if ($is_master == -1) {
			# An appropriate error message would
			# already have been logged in the
			# is_master sub
			return 1;
		}
		elsif ($is_master == 0) {
			# Slave node
			# Dont do anything
			return 0;
		}
	}

	#
	# IMPORTANT: Control should come here only if:
	# This is a non-RAC cluster
	# Or
	# This is a RAC cluster and this system is the CVM Master
	#

	my $ret = 0;
	my $vcheck=0;
	my ($oto) = split('\n', `$ag_i18n_inc::HATYPE -value SRDFSnap OnlineTimeout`);
	my $count = int ($oto / 10);
	
	if (!$self->{'instant'}) {
		if ($self->{'dcv_mode'} eq "Asynchronous") {
			push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -consistent verify");
		} else {
			push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -synchronized verify");
		}
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "R1-R2 are not synchronized or consistent.\n", 140);
			return 1;
		}
	}

	if ($self->{'copymode'} eq "clone") {
         $vcheck = is_SYMCLI_version_6();

         #version 7 & above
		if( $vcheck == 0)
		{
			push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} $self->{'tgtflag'} create -nocopy -noprompt ");
		}
		else {
			push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} $self->{'tgtflag'} create -noprompt ");
		}

		push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -created verify");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Copy session is not created for $self->{'group'}\n", 139, $self->{'group'});
			return 1;
		}
	}
	elsif ($self->{'copymode'} eq "snap") {
		if ($self->{'savepoolname'} ne "") {
			push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} create -svp $self->{'savepoolname'} -noprompt");
		} else {
			push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} create -noprompt");
		}
		push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} -created verify");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Copy session is not created for $self->{'group'}\n", 139, $self->{'group'});
			return 1;
		}
    } elsif ($self->{'copymode'} eq "mirror") {
		# Verify that BCV are synced
		push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -synched verify");
        #verify that R1-R2 are synced
	}

	if ( $self->{'copymode'} eq "clone" ) {
		push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} $self->{'tgtflag'} activate -nop");
		$ret = ar_run_cmds(\@cmds);
		#Once activated group can be in one of two modes copyonaccess or copyonwrite
		push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -copyonaccess verify");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -copyonwrite verify");
		    $ret = ar_run_cmds(\@cmds);
		}
		if ($ret) {
			VCSAG_LOG_MSG("E", "Clone session is not activated for $self->{'group'}\n", 141, $self->{'group'});
			return 1;
		}
	}
	elsif ($self->{'copymode'} eq "snap") {
		push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} activate -nop");
		push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} -copyonwrite verify");
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Copy session is not activated for $self->{'group'}\n", 141, $self->{'group'});
			return 1;
		}
	} elsif ($self->{'copymode'} eq "mirror") {
		push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -noprompt -i 5 -c $count -consistent split");
		push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -split verify");

		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
            VCSAG_LOG_MSG("E", "Couldn't take the consistent snapshot and hence will have to exit", 104);
            return 1;
        }
    }

    return 0;
}

sub split_replication {
    my $self = shift;
	my $is_rac_cluster = $self->{'rac'};
	my $is_master = 0;
	my $ret = 0;
	my @cmds = ();

	if ($is_rac_cluster == 1) {
		# This is a RAC environment i.e. the group for which a 
		# FireDrill is being run is a parallel group.
		$is_master = ArrayRep::is_cvm_master();

		if ($is_master == -1) {
			# An appropriate error message would
			# already have been logged in the
			# is_master sub
			return 1;
		}
		elsif ($is_master == 0) {
			# Slave node
			# Dont do anything
			return 0;
		}
	}

	#
	# IMPORTANT: Control should come here only if:
	# This is a non-RAC cluster
	# Or
	# This is a RAC cluster and this system is the CVM Master
	#

    my $ret = 0;
	my ($oto) = split('\n', `$HATYPE -value SRDFSnap OnlineTimeout`);
	my $count = int($oto / 10);

	if ($self->{'dcv_mode'} eq "Asynchronous") {
		push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -i 5 -c $count -consistent verify");
        push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -noprompt -i 5 -c $count -force split");
	} else {
		push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -i 5 -c $count -synchronized verify");
        push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -noprompt -i 5 -c $count split");
	}
	push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -split verify");

	#
	# if cannot split, exit with error
	#
    $ret = ar_run_cmds(\@cmds);
	if ($ret) {
		VCSAG_LOG_MSG("E", "Cannot split replication; Exiting", 118);
		return 1;
	}
	return $ret;
}

#
#Find Vxdiskname/Vxdmpnode for the given Symmetrix Array and the Device number
#Uses syminq command to correlate the symmetrix device with the vxdisk/vxdmp node names.
#
sub findvxdiskname
{
	my ($self) = shift;
	my ($ArraySno) = shift;
	my ($devid) = shift;
	my $cmdoutput_ref = shift;
	my $vxdiskpath_op_ref = shift;
	my @cmdoutput = @$cmdoutput_ref;
	my @vxdiskpath_op = @$vxdiskpath_op_ref;

	foreach my $l (@cmdoutput) {
		if ($l =~ /$devid/ ){
			my (@fields) = split('\s+', $l);
			my $ArraySno2 = $fields[0];
			my $devpath = $fields[1];
			my $devid1 = $fields[2];
			if ($ArraySno2  eq $ArraySno)
			{
				my ($OSname)  = (POSIX::uname())[0];
				if ($devpath =~ /^\Q$VXPATH\E/)
				{
					my (@pathfields) = split('/',$devpath);
					if ( $OSname eq "SunOS" )   # Snip trailing "s2"  for Solaris
					{
						$pathfields[4] =~ s/s2$//;
					}
					return $pathfields[4];
				} else {
					VCSAG_LOGDBG_MSG(1, "findvxdiskname: osh: [$devpath]. No DMP node. Trying to find vxdisk name.");
					if ($devpath =~ /(\w+)$/) {
						my $osh = $1;
						if ( $OSname eq "SunOS" )   # Snip trailing "s2"  for Solaris
						{
							$osh =~ s/s2$//;
						} elsif ($OSname eq "AIX") {
							$osh =~ s/^r//;
						}
						my @lines = grep(/^$osh\W+/, @vxdiskpath_op);
						my $num = @lines;
						if ($num == 0) {
						# Additional grep required because on solaris 10 name of the device has s2 as a suffixx
							@lines = grep(/^$osh/, @vxdiskpath_op);
							$num = @lines;
							if ($num == 1) {
								$lines[0] =~ /^$osh\w+\s+(\w+)/;
								my $vxdisk = $1;
								return $1;
							}
						}
						if ($num != 1) {
							VCSAG_LOGDBG_MSG(1, "findvxdiskname: Multiple matches!");
							next;
						}
						$lines[0] =~ /^$osh\s+(\w+)/;
						my $vxdisk = $1;
						return $vxdisk;
					}
				}
            }
        }
    }
    return "none found";
}

#
# For vxvm as the underlying volume manager, the subroutine will clear
# import information on the disks in the device group, update the udid
# of the disks (if it is a rac environment), and set an appropriate tag
# on the disks so that when the vxdg import cmd is fired with the
# appropriate cli options, it will succeed.
# Returns: 0 on success
#	   1 on failure
#

# get_disks returns the list of disks involved in the DiskGroup configured above SRDF
# This function executes two command `syminq -pdevfile` and `vxdisk path`
# 

sub get_disks {
	my ($self) = shift;
	my $disk_ref = shift;
	my $searchdev = shift;
	my $ret = 0;
	my $ArraySno= " ";
	my (@disks, @lines);
    my (@syminq_output, @vxdiskpath_op);
	
    #Store syminq command output
    if ($osType !~ /^MSWin/i) {
        (@syminq_output) = split('\n', `$SYMINQ -pdevfile`);
        (@vxdiskpath_op) = split('\n', `$VXDISK path`);
    }
	#
	# First get the R2 device names from sym
	#
	if($self->{'iscompositegroup'} == 0) {
		(@lines) = split('\n', `$SYMDG show $self->{'group'}`);
	} else {
		(@lines) = split('\n', `$SYMCG show $self->{'group'}`);
	}
	foreach my $l (@lines) {
		if ($l =~ /Symmetrix\sID/) {  
		    if (!($l =~ /Remote\sSymmetrix\sID/)) {
				my (@fields2) = split (':',$l);
				$ArraySno = $fields2[1];
                # Trim Array Serial Number
				$ArraySno =~ s/^\s+//;
				$ArraySno =~ s/\s+$//;
		    }
		}
		elsif ($l !~ /$searchdev/) { next; }
		my (@fields) = split('\s+', $l);
		if (!($fields[1] =~ /^$searchdev/)) { next;}
        my ($vxdiskname, $symdevid);
        if ($osType =~ /^MSWin/i) {
            $vxdiskname = $fields[2];
            $vxdiskname =~ s/.*\///;
            $vxdiskname =~ s/\\\\.\\PHYSICALDRIVE/Harddisk/;
        } else {
            $symdevid = $fields[3];
            $vxdiskname = $self->findvxdiskname($ArraySno,$symdevid,\@syminq_output,\@vxdiskpath_op);
        }
		if ($vxdiskname ne 'none found') {
            push(@{$disk_ref}, $vxdiskname);
        } else {
			# Failed to get the dmpnodename of the device
			VCSAG_LOG_MSG("E", "Failed to get the dmpnodename for device $symdevid, exiting",
                    115, "$symdevid");
			return 1;
 		}
	}
	return 0;
}


# Returns the TAG based on the $case supplied in the Argument.
#sub also used for dev_scan action script.

sub get_tag {
	my $self = shift;
	my $case = shift;
	my $searchdev = shift;
	my $tag = shift;

	$$searchdev = "BCV\\d";
	$$tag = "BCV-";

	if ($self->{'tgtflag'} && $self->{'copymode'} eq "clone") {
		$$searchdev = "TGT\\d";
		$$tag = "TGT-";
	}
	if ($self->{'copymode'} eq "snap") {
		$$searchdev = "VDEV\\d";
		$$tag = "VDEV-";
	}

        if ($case !~ /gold/) {
		#
		# For silver and bronze cases
		#
		$$searchdev = "DEV\\d";
		$$tag = "DEV-";
	}
}

# Performs an offline operation on the disks which were tagged using "vxdisk offline" command.

sub offline_disks {
	if ($OS eq "Windows NT") {
		return;
	}
	my $self = shift;
	my $case = shift;
	my ($searchdev, $tag, @cmds);
	$self->get_tag ($case, \$searchdev, \$tag);
	my $final_tag = $tag.$self->{'group'};	
	my @listtag = split (/\n/, `$VXDISK listtag`);
	my @tagged_disks = grep (/\b$final_tag\b/, @listtag);	
	for my $disk (@tagged_disks) {
		my @line = split (/\s+/, $disk);
		push (@cmds, "$VXDISK offline $line[0]");
		my $ret = ar_run_cmds(\@cmds);
       	if ($ret) {
       		VCSAG_LOG_MSG ("E", "vxdisk offline command failed for disk [$line[0]]", 153, "$line[0]");
       	}
	}
}

# Performs an online operation on the disks using "vxdisk online" command.
# Refrence to the array of disk is passed as argument.

sub online_disks {
	my $self = shift;
	my $disk_ref = shift;
	my @cmds = ();

	for my $disk (@{$disk_ref}) {
		push (@cmds, "$VXDISK online $disk");
		my $ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG ("E", "vxdisk online command failed for disk [$disk]", 152, "$disk");
		} 
	}
}

sub srdf_prepare_disks {
	my ($self) = shift;
	my ($case) = shift;
	my ($tagname) = shift; # We should be able to return the 
				# tagname value to the caller this way
				# tagname is a pass-by-reference argument
	my $ret = 0;
	my $ArraySno= " ";
	my (@disks, @lines, $searchdev, $tag);
	
	my ($is_rac_cluster) = $self->{'rac'};
       
	$self->get_tag ($case, \$searchdev, \$tag);

	$ret = $self->get_disks(\@disks, $searchdev);
	if ($ret){
		# Error message already logged in get_disks
		return 1;
	}
			

	# TBD unique tagname across clusters

	if ($OS ne "Windows NT") {	
		$self->online_disks (\@disks);
	}
	$$tagname = $tag . $self->{'group'};
	my $final_tag = $tag . $self->{'group'};

	# In the future, there should be code to check the type of
	# volume manager technology before calling the appropriate
	# volume-manager-specific routine to set up the disks for an
	# import operation.
	$ret = vxvm_prepare_disks($final_tag, $is_rac_cluster, @disks);

	return $ret;
}

#
# The subroutine sets up everything for a successful import of the
# diskgroup on the right set of disks: BCV-luns if gold firedrill and
# DEV-luns in case of silver and bronze firedrills.
# Further, if CVM is the underlying volume manager being used, the
# subroutine will make sure that the disks setup and dg import happens
# only on the CVM MASTER.
# Return Values:
# --------------
# 0 on success
# 1 on failure
#
sub mount_vxvm {
	my ($self, $case) = @_;
	my $ret = 0;
	my $is_master = 0;
	my $is_rac_cluster = $self->{'rac'};

    scan_devs();
	if ($is_rac_cluster == 1) {
		# This is a RAC environment i.e. the group for which a 
		# FireDrill is being run is a parallel group.
		$is_master = ArrayRep::is_cvm_master();

		if ($is_master == -1) {
			# An appropriate error message would
			# already have been logged in the
			# is_master sub
			return 1;
		}
		elsif ($is_master == 0) {
			# This node is the SLAVE
			# Wait in a loop until the MASTER 
			# imports the shared dg
			$ret = ArrayRep::cvm_slave_wait_for_shared_dg_import($self->{'dgname'});
			if (!$ret) {
				# Dg has been imported by the 
				# MASTER - so all shd work ok
				return 0;
			} else {
				# An appropriate error message would
				# already have been logged in the
				# slave_wait_for_shared_dg_import sub
				return 1;
			}
		}
	}

	#
	# COME HERE ONLY IF THE NODE IS THE CVM MASTER OR THIS IS A
	# NON-RAC ENVIRONMENT
	#

	# Clear any import info, update udid, set tag on the appropriate
	# disks. Basically setup the disks for the impending dg import
	my $tagname = "";

	# tagname will be passed by reference into the prepare_disks sub
	#
	# Setup the disks for the impending dgs import
	#
	$ret = $self->srdf_prepare_disks($case, \$tagname);

	if ($ret) {
		# An appropriate error message would
		# already have been logged in the
		# sub prepare_disks
		return 1;
	}

	#
	# Test import of the dgs
	#
	$ret = $self->vxvm_test_diskgroup_import($self->{'dgname'}, $case, $tagname, $is_rac_cluster);
	return $ret;
}


sub mount_snapshot_lvm {
	my ($self, $case) = @_;

	my $ret = 0;
	my @cmds = ();
	my (@bcvdisks, @lines);
	my ($vgname, $disk);

	my $searchdev = "BCV0";

	# Map VG names to VG ids
	my %vgs_to_id = ();

	# Map VGid to all its BCV disks
	my %vgid_to_disks = ();

	# First get the R2 device names from sym
	if($self->{'iscompositegroup'} == 0) {
		(@lines) = split('\n', `$SYMCG show $self->{'group'}`);
	} else {
		(@lines) = split('\n', `$SYMDG show $self->{'group'}`);
	}
	foreach my $l (@lines) {
		if ($l !~ /$searchdev/) { next; }
		my (@fields) = split('\s+', $l);
		my $devbcv = $fields[2];
		$devbcv =~ s/.*\///;
		$devbcv =~ s/^r//;     # if device name is a raw device, convert it
		push(@bcvdisks, $devbcv);
	}

	my @vgnames = split(' ', $self->{'dgname'});

	# Construct a hash to map VGname to VG serial id
	ArrayRep::aix_lvm_get_vgs_id(\@vgnames, \%vgs_to_id);

	my @vgids = (values %vgs_to_id);

	# Construct a hash to map VGids to bcv disks
	ArrayRep::aix_lvm_get_vgdisks(\@vgids, \%vgid_to_disks, \@bcvdisks);

	foreach $vgname (keys %vgs_to_id) {
		my $Lv_Prefix = "fd_";

		my $fdvgname = $vgname . "_fd";
		my @disks = @{$vgid_to_disks{$vgs_to_id{$vgname}}};

		foreach $disk (@disks) {
			push(@cmds, "$CHDEV -l $disk -a pv=clear");
			$ret = ar_run_cmds(\@cmds);

			if ($ret) {
				VCSAG_LOG_MSG( "W",
		"Could not clear PV id on $disk. recreatevg might fail",
					134, "$disk" );
			}
		}

		#push(@cmds, "$RECREATEVG -y $fdvgname -Y fd_ -L /fd -n @disks" );
	
		for (my $ii = $RECREATEVGTIMER; $ii > 0; $ii--) {
			push(@cmds, "$RECREATEVG -y $fdvgname -Y $Lv_Prefix -n @disks");
			$ret = ar_run_cmds(\@cmds);
			$Lv_Prefix = "NA";
			last if (!$ret);
			VCSAG_LOG_MSG("I",
                    "recreatevg failed; retrying again after $RECREATEVGINTVL seconds; $ii more retries left",
				137, "$RECREATEVGINTVL", "$ii");
			push(@cmds, "sleep $RECREATEVGINTVL");
			ar_run_cmds(\@cmds);
		}

		if ($ret) {
			VCSAG_LOG_MSG("E",
				"Cannot recreate VG $fdvgname; exiting",
				123, "$fdvgname");
			return 1;
		}

		my $line = `$ag_i18n_inc::HARES -list VolumeGroup=$fdvgname`;
		my ($lvmresname, undef) = split(/\s+/, $line);

		ArrayRep::aix_lvm_update_majornum_attrib($lvmresname, \@disks);
	}

	return 0;
}

#
# Do a resynch in the gold firedrill case.
# In this case, establish the relationship between the target devices
# and the bcvs
# If the firedrill was a fall-through to the bronze case, establish the
# relationship between the devices on the primary and secondary sites.
#
sub resynch_gold {
	my ($self) = @_;
	my $ret = 0;
	my @cmds = ();

	my ($oto) = split('\n', `$HATYPE -value SRDF OnlineTimeout`);
	my $count = int($oto / 10);
	if ($self->{'vmtype'} eq "LVMVG") {
		my @dgs = map($_ . "_fd", (split(/\s+/, $self->{'dgname'})));
		$ret = ArrayRep::aix_lvm_exportvgs(\@dgs);
	}

	my ($took) = split('\n', `$ag_i18n_inc::HARES -value $self->{'resname'} Responsibility`);
	if ($took) {
		if (!$self->{'instant'} && ($took eq "bronze")) {
# This means bronze-case..
# to take-care of fall-through bronze case.
			$ret = $self->resynch_bronze();
            return $ret;
		}
		if ( $self->{'copymode'} eq "clone" ) {
			push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -copyonaccess verify");
			$ret = ar_run_cmds(\@cmds);
			if ($ret) {
				push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -copyonwrite verify");
				$ret = ar_run_cmds(\@cmds);
			}
			push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} $self->{'tgtflag'} terminate -nop");
		} elsif ($self->{'copymode'} eq "snap") {
			push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} -copyonwrite verify");
			push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} terminate -noprompt" );
		} elsif ($self->{'copymode'} eq "mirror") {
			push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -split verify" );
			push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -force -noprompt -i 5 -c $count establish" );
		}
		$self->offline_disks("gold");
		$ret = ar_run_cmds(\@cmds);
		`$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility ""`;
		if ($ret) {
			VCSAG_LOG_MSG("E", "cannot resynch required snapshot of group $self->{'group'}; exiting", 121, "$self->{'group'}");
			return 1;
		}
	} else {
		VCSAG_LOG_MSG("I", "Nothing to resynch", 146);
	}

	return 0;
}

sub resynch_silver {
	my ($self) = @_;
	my $ret = 0;
	my @cmds = ();

	my ($oto) = split('\n', `$HATYPE -value SRDF OnlineTimeout`);
	my $count = int($oto / 10);
	if ($ret) {
		VCSAG_LOG_MSG("E", "cannot resynch group $self->{'group'}; exiting", 120, "$self->{'group'}");
		return 1;
	}

	my ($took) = split('\n', `$ag_i18n_inc::HARES -value $self->{'resname'} Responsibility`);
	if ($took) {
		if ($took eq "bronze") {
# This means bronze-case..
# to take-care of fall-through bronze case.
			$ret = $self->resynch_bronze();
            return $ret;
		}
    	if ( $self->{'copymode'} eq "clone" ) {
        push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -copyonaccess verify");
		$ret = ar_run_cmds(\@cmds);
			if ($ret) {
				push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} -copyonwrite verify");
		    	$ret = ar_run_cmds(\@cmds);
			}
			push(@cmds, "$SYMCLONE $self->{'grptype'} $self->{'group'} $self->{'tgtflag'} terminate -nop");
		} elsif ($self->{'copymode'} eq "snap") {
			push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} -copyonwrite verify");
			push(@cmds, "$SYMSNAP $self->{'grptype'} $self->{'group'} terminate -noprompt" );
    	} elsif ($self->{'copymode'} eq "mirror") {
			push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -split verify");
			push(@cmds, "$SYMMIR $self->{'grptype'} $self->{'group'} -noprompt -i 5 -c $count establish");
		}
		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "cannot resynch required snapshot of group $self->{'group'}", 122, "$self->{'group'}");
		} 
		push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -split verify");
		push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -noprompt -i 5 -c $count establish");
		$self->offline_disks("silver");
		$ret = ar_run_cmds(\@cmds);
		`$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility ""`;
		if ($ret) {
			VCSAG_LOG_MSG("E", "cannot resynch group $self->{'group'}; exiting", 120, "$self->{'group'}");
			return 1;
		}
	} else {
		VCSAG_LOG_MSG("I", "Nothing to resynch", 146);
	}
	return 0;
}

sub resynch_bronze {
	my ($self) = @_;
	my $ret = 0;
	my @cmds = ();

	my ($oto) = split('\n', `$HATYPE -value SRDF OnlineTimeout`);
	my $count = int($oto / 10);
	
	my ($took) = split('\n', `$ag_i18n_inc::HARES -value $self->{'resname'} Responsibility`);
	if ($took) {
		push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -split verify");
		push(@cmds, "$SYMRDF $self->{'grptype'} $self->{'group'} -noprompt -i 5 -c $count establish");
		$self->offline_disks("bronze");
		$ret = ar_run_cmds(\@cmds);
		`$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility ""`;
		if ($ret) {
			VCSAG_LOG_MSG("E", "cannot resynch group $self->{'group'}; exiting", 120, "$self->{'group'}");
			return 1;
		}
	} else {
		VCSAG_LOG_MSG("I", "Nothing to resynch", 146);
	}
	return 0;
}

sub resynch {
	my ($self, $type) = @_;
	my $ret = 0;

	if ($type eq "SRDF"){
# Get the IsCompositeGroup flag from the Target.
		my $cmd = "$ag_i18n_inc::HARES -value $self->{'target'} IsCompositeGroup";
		($self->{'iscompositegroup'}) = `$cmd`;
		$self->{'grptype'} = ($self->{'iscompositegroup'} == 1) ? " -cg " : " -g ";

# Set the SYMAPI_SERVER env variable if defined on the target object.
            
		SRDFAgent::get_symapi_servers($self->{'target'}, $self->{'basedir'});
                if ($ENV{SYMCLI_CONNECT}) {
                 if (($self->{"vmtype"} eq "LPAR" ) && $self->{"sotype"} eq "gold-only") {
                   VCSAG_LOGDBG_MSG(1, "Remote SymAPI server is configured with LPAR and gold only fire-drill.") 
                 } else {
			VCSAG_LOG_MSG("E", "SRDF FireDrill is not supported when Remote Symapi Servers are configured.", 50);
			return 1;
                 }
		}
	} elsif (exists ($vmattrib{$type})) {
		$self->{'grptype'} = "-g";
	}

	if ($self->{'sotype'} =~ /gold/) {
		$ret = $self->resynch_gold();
	} elsif ($self->{'sotype'} =~ /silver/) {
		$ret = $self->resynch_silver();
	} else {
		$ret = $self->resynch_bronze();
	}
	return $ret;
}

sub pitc_gold {
	my ($self) = @_;
	my $ret = 0;
	my $snapret = 0;

	$snapret = $self->take_snapshot();
	if ($snapret) {
		if ($self->{'sotype'} eq "gold-only") {
			VCSAG_LOG_MSG("E", "cannot take required snapshot of group $self->{'group'}; exiting", 106, "$self->{'group'}");
			return 1;
		} else {
			VCSAG_LOG_MSG("W", "cannot take required snapshot of group $self->{'group'}; dropping to bronze type", 108, "$self->{'group'}");
			$ret = $self->pitc_bronze();
			if ($ret == 0) {
		        `$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility bronze`;
      		}
			return $ret;
		}
	} else {
		`$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility gold`;
	}

	if ($self->{'vmtype'} eq "LVMVG") {
		$ret = $self->mount_snapshot_lvm("gold");

	} elsif ($self->{'vmtype'} eq "LPAR" && $self->{"sotype"} eq "gold-only") {
		VCSAG_LOGDBG_MSG(1, "firedrill is configured with LPAR resource."); 
	}
	else {
		$ret = $self->mount_vxvm("gold");
	}

	return $ret;
}

sub pitc_silver {
	my ($self) = @_;
	my $ret = 0;
	my $snapret = 0;
	
	$snapret = $self->take_snapshot();
	if ($snapret) {
		if ($self->{'sotype'} eq "silver-only") {
			VCSAG_LOG_MSG("E", "cannot take required snapshot of group $self->{'group'}; exiting", 106, "$self->{'group'}");
			return 1;
		} else {
			VCSAG_LOG_MSG("W", "cannot take required snapshot of group $self->{'group'}; dropping to bronze type", 108, "$self->{'group'}");
			$ret = $self->pitc_bronze();
			if ($ret == 0) {
		        `$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility bronze`;
      		}
			return $ret;
		}
	} else {
		`$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility silver`;
	}
	$ret = $self->split_replication();
	if ($ret) {
		VCSAG_LOG_MSG("E", "Could not Split the replication in pitc_silver for $self->{'group'};", 107, "$self->{'group'}");
		return 1;
	}

	if ( $self->{'vmtype'} ne "LVMVG" ) {
		$ret = $self->mount_vxvm("silver");
	}

	return $ret;
}

sub pitc_bronze {
	my ($self) = shift;
	my $ret = 0;

	VCSAG_LOG_MSG("N", "\n\nBRONZE:Performing read-write replication split\n\n", 105);
	# perform read-write replication split
	# insert command to split replication and write enable target
	$ret = $self->split_replication();
	if ($ret) {
		VCSAG_LOG_MSG("E", "Could not Split the replication in pitc_bronze for $self->{'group'};", 109, $self->{'group'});
		return 1;
	} else {
		`$ag_i18n_inc::HARES -modify $self->{'resname'} Responsibility bronze`;
	}	

    if ($self->{'vmtype'} ne "LVMVG") {
		$ret = $self->mount_vxvm("bronze");
	}
	VCSAG_LOG_MSG("N", "\n\nBRONZE:Processing completed for group $self->{'group'}\n\n", 110, "$self->{'group'}");
	return $ret;
}

sub pitc {
	my ($self, $type) = @_;
	my $ret = 0;

	if ($type eq "SRDF") {
# Get the IsCompositeGroup flag from the Target.
		my $cmd = "$ag_i18n_inc::HARES -value $self->{'target'} IsCompositeGroup";
		($self->{'iscompositegroup'}) = `$cmd`;
		$self->{'grptype'} = ($self->{'iscompositegroup'} == 1) ? " -cg " : " -g ";
	} elsif (exists ($vmattrib{$type})) {
		$self->{'grptype'} = "-g";
	}
	$ret = $self->get_mode($type);
	if(!$ret) {
		if ($self->{'sotype'} =~ /gold/) {
			$ret = $self->pitc_gold();
		} elsif ($self->{'sotype'} =~ /silver/) {
			$ret = $self->pitc_silver();
		} else {
			$ret = $self->pitc_bronze();
		}
	}
	return $ret;
}

sub reenable_vxconfigd_on_slaves {
	my $self = shift;
	my $is_rac_cluster = $self->{'rac'};
	my $is_master = 0;
	my (@v1) = ();
	my $nodename = "";
	my $ret = 0;
	my $reenable_failed = 0;

	my ($grp) = split('\n', `$HARES -value $self->{'resname'} Group`);
	my ($par) = split('\n', `$HAGRP -value $grp Parallel`);
	if ($par != 1) {
		return 0;
	}
	my ($sup) = split('\n', `$HATYPE -value CVMVolDg SupportedActions`);
	if (! ($sup =~ /vxdctlenable/) ) {
		return 0;
	}

	if ($is_rac_cluster == 1) {
		# RAC cluster - slave does nothing here
		$is_master = ArrayRep::is_cvm_master();

		if ($is_master == -1) {
			return 1;
		}
		elsif ($is_master == 0) {
			# Slave node: dont do anything
			return 0;
		}
	}

	# get the list of all nodes in the cvm cluster
	my (@vxclus) = split('\n', `$VXCLUSTADM nidmap 2>$DEVNULL`);
	foreach my $v (@vxclus) {
		if (($v !~ /Master/) && ($v !~ /^Name/)) {
			# for all slave nodes
			(@v1) = split('\s+', $v);
			$nodename = $v1[0];
			$ret = $self->reenable_vxconfigd($nodename);
			if ($ret == 1) {
				$reenable_failed = 1;
			}
			$self->online_disk_on_slaves($nodename);
		}

	}
	if ($reenable_failed == 1) {
		VCSAG_LOG_MSG("W",
	"The action to reenable the volume configuration daemon failed on atleast one slave node in the CVM cluster. This may cause the CVM diskgroup to not be imported successfully.", 113);
	}
}

sub run_cmd_with_output {
	my ($cmd, $output_ref, $ignore_failure) = @_;
	my $retval = 1;

	if (1 == $ignore_failure){
		@{$output_ref} = `$cmd 2>$DEVNULL`;
	} else {
		@{$output_ref} = `$cmd`;
	}

	if ($? != 0) {
		$retval = 0;
		unless (1 == $ignore_failure ) {
			VCSAG_LOG_MSG("W", "Command: \'$cmd\' failed !", 1007,
					"$cmd");
		}
	} else {
		chomp @{$output_ref};
	}

	return $retval;
}

sub FireDrillDiskGroupName {
	#Name of diskgroup used by application group. This is input
	my ( $self, $currentDGName ) = @_;

	# Name of the firedrill diskgroup.
	my $firedrillDGName;

	my $userSuppliedListOfFDDiskgroupName;

	my $standardFDName = $currentDGName . "_fd";

	# Check if the newly added attribute is specified.
	# We are assuming the name of this attribute same in all agents.
	my $newattributeName = "DiskGroupSnapList";

	# Call the HARES and get values.
	($userSuppliedListOfFDDiskgroupName) = split('\n',`$ag_i18n_inc::HARES -value $self->{'resname'} $newattributeName`);
	if ( $userSuppliedListOfFDDiskgroupName eq ""){
		# either the values are not provided or attribute is not present
		# return standard name
		# should we log this.
		return $standardFDName;
	}

	# the format is in DGName;FD_DGName and then separated by space.
	# sanjay, we can change format!

	my (@dg_fddg_NameMap) = split(' ', $userSuppliedListOfFDDiskgroupName);
	foreach my $dg_fddg_NameEntry (@dg_fddg_NameMap) {
		#check till the ;, so that we do not search the portion of the name
		my (@fdDiskGroupName) = split(';', $dg_fddg_NameEntry);
		if ( $fdDiskGroupName[0] eq $currentDGName){
			# found it.
			$firedrillDGName = $fdDiskGroupName[1];
			return $firedrillDGName;
		}
	}
	# well, nothing worked so fall back to existing convention.
	VCSAG_LOG_MSG("W", "Diskgroup [$currentDGName] not specified in DiskGroupSnapList. Using standard name [$standardFDName] for importing the Diskgroup.", 150, "$currentDGName", "$standardFDName");
	return $standardFDName;
}

sub reenable_vxconfigd {
	my ($self, $nodename) = @_;
	my $reenable_failed = 0;

	# get the resources that depend on the srdfsnap resource
	my (@deps) = split('\n', `$ag_i18n_inc::HARES -dep $self->{'resname'}`); 

	foreach my $d (@deps) {
		if ($d =~ /#Group/) {
			next;
		}
		my (@fields) = split('\s+', $d);
		if ($fields[2] ne $self->{'resname'}) {
			next;
		}
		my ($typ) = split('\n', `$ag_i18n_inc::HARES -value $fields[1] Type`);
		if ($typ eq "CVMVolDg") {
			# Invoke the action to run vxdctl enable on the slave 
			# node. It is sufficient to invoke the action for just
			# one CVMVolDg resource on a node, if the action
			# succeeds
			`$ag_i18n_inc::HARES -action $fields[1] vxdctlenable -sys $nodename`;
			$reenable_failed = $?;
			if (!$reenable_failed) {
				# The action succeeded
				last;
			}
		}
	}
	return $reenable_failed;
}

#
# scandev - Scan devices after a snapshot has been taken to detect the newly visible snapshot luns
#

sub scan_devs {
	my ($host) = @_;
	my $cmd;
	my $ret;

	my ($SYSTEM) = (POSIX::uname())[0];
    return 0 if (($SYSTEM eq "Windows NT") or ($SYSTEM eq "Linux"));

	if ($SYSTEM eq "AIX")
	{
		$cmd="$CFGMGR";
	}
	elsif ($SYSTEM eq "HP-UX")
	{
		$cmd="$IOSCAN -fnC disk 2>&1 > $DEVNULL";
	}
	elsif ($SYSTEM eq "SunOS")
	{
		## Runs devfsadm  and vxdctl enable
 		$cmd="$VXDISKCONFIG";
	}

	VCSAG_LOG_MSG("I", "Executing command [$cmd] ", 132, "$cmd");
	`$cmd`;
    if ($? != 0) {
		$ret = "$?";
		VCSAG_LOG_MSG("E", "Command [$cmd] failed with error code: $ret", 101, "$cmd", "$ret");
		return 1;
	}

	return 0;
}


#
# Arguments to the subroutine:
# ----------------------------
# dgname: The string comprising of the original diskgroups names
# (without the _fd appended to the names) separated by a " "
# case: Type of Firedrill: gold/silver/bronze
# tagname: The value of the tag set on the disks on which the diskgroup
#	is to be imported
# is_rac_cluster: 1 if this is a RAC environment in which the agent is
#	running; 0 otherwise
#
# Return Values:
# --------------
# 0 on success;
# 1 otherwise and in this case, an appropriate error message is logged
#
# Functionality:
# --------------
# This subroutine checks that after the disks have been setup right,
# the diskgroups can be successfully imported on the system. If the
# subroutine encounters error in attempting to import/deport the
# diskgroups, it returns a failure code (1) along with logging the
# appropriate error message.
#
sub vxvm_test_diskgroup_import {

    my $self = shift;
	my ($dgnamelist, $case, $tagname, $is_rac_cluster) = @_;
	my @dglist = split(' ',$dgnamelist);
	my $newdg;
	my $options = "";
	my $rac_options = "";

	#
	# $options contains the options to be used during the diskgroup
	# import in a non-RAC env whereas $rac_options contains the
	# options to be used during the diskgroup import in a RAC env
	#

	my (@cmds) = ();
	my $ret = 0;

	if ($case eq "gold") {
		#
		# GOLD firedrill
		#
		$options = "-C -o useclonedev=on -o updateid -o tag=$tagname";
		$rac_options = "-s -C -o updateid -o tag=$tagname";
	} else {
		#
		# BRONZE or SILVER Firedrill
		#
		$options = "-C -o updateid";
		$rac_options = "-s -C -o updateid";
	}

	foreach my $dgname (@dglist) {

		#following line is original
	    $newdg = "$dgname" . "_fd";
		$newdg = FireDrillDiskGroupName($self, $dgname);
		#
		# Import the dg as a non-shared dg first. This is a
		# workaround for a bug in CVM. Refer to etrack incident
		# 1053529 for more info on this bug.
		#
		if ($OS ne "Windows NT") {
			push(@cmds, "$VXDG $options -n $newdg import $dgname");
		} else {
			push(@cmds, "$VXDG $options -n $newdg import -g $dgname");
		}

		$ret = ar_run_cmds(\@cmds);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Cannot import disk group $dgname in pitc $case; exiting",
				60003, "$dgname", "$case");
			return 1;
		}
		if ($OS ne "Windows NT") {
			push(@cmds, "$VXDG deport $newdg");
			$ret = ar_run_cmds(\@cmds);
			if ($ret) {
				VCSAG_LOG_MSG("E", "Cannot deport disk group $newdg in pitc $case; exiting", 60004, "$newdg", "$case");
				return 1;
			}
		}
	}

	return 0;
}

#
# This subroutine prepares disks assuming VxVM to be the volume manager
# technology. This subrtoutine is called from the array-based
# technology specific prepare_disk subroutine.
# Arguments to the subroutine:
# ----------------------------
# disks: List of disks that the device group has been created upon. To
#	get this list of disks, run the technology specific command to view
#	the device group and the disks underlying it.
# tagname: The tagname to be used to tag the specific volume manager disks.
#	This is the tag that will be used to import the disk group on these
#	disks.
# is_rac_cluster: 1 if this is a RAC environment in which the agent is
#	running; 0 otherwise
# Return Value:
# -------------
# 0 on success;
# 1 otherwise. An appropriate error message is logged by this
#	subroutine when an error is encountered
# Functionality:
# --------------
# This subroutine does the following tasks:
#	1. Gets the volume manager disks that map to the disks the device
#	group is created on
#	2. For each volume manager disk that comprises the device group, the
#	subroutine does:
#		vxdisk clearimport to clear import information on the volume
#		  manager disks
#		vxdisk settag <tagname> to set a tag to uniquely identify the
#		  disks during an import operation
#	If any of these operations fail, the subroutine logs an
#	appropriate error message and returns 1 to indicate a failure.
#
sub vxvm_prepare_disks {
    if ($OS ne "Windows NT") {
        return ArrayRep::vxvm_prepare_disks(@_);
    }
	my ($tagname, $is_rac_cluster, @disks) = (@_);
	my (%vxdisk) = ();
	my (@cmds) = ();
	my $ret = 0;
	my $iter = 0;
	VCSAG_LOGDBG_MSG(1,
		"is_rac_cluster is $is_rac_cluster; tag is $tagname");

	VCSAG_LOG_MSG("I", "Issuing command $VXASSIST rescan", 151, $VXASSIST); 
  	my (@Dellines) = split('\n', `$VXASSIST rescan`);

  	my @lines;
	(@lines) = split('\n', `$VXDISK list`);

	foreach my $l (@lines) {
		my (@fields) = split('\s+', $l);
		$vxdisk{$fields[$#fields]} = $fields[0];
	}

	foreach my $disk (@disks) {
		if ($vxdisk{$disk}) {
			$disk = $vxdisk{$disk};
		}
 #windows VM has not implemented or changed follwing.
		$iter = 0;
		$ret = 1;
		while ( ($iter < 2) && ($ret != 0) ){
			push(@cmds, "$VXDISK set tag value=$tagname $disk");
			$ret = ar_run_cmds(\@cmds);
      			$iter = $iter + 1;
    		}
    		if ($ret) {
			VCSAG_LOG_MSG("E",
				"Could not tag disk $disk; exiting;",
				60012, "$disk");
			return 1;
		}
	}
	# All has succeeded
	return 0;
}

sub ar_run_cmds {
    if ($OS ne "Windows NT") {
        return ArrayRep::ar_run_cmds(@_);
    }
	my ($CMDSref) = @_;
	my $ret = 0;
	foreach my $cmd (@{$CMDSref}) {
		$ret = ar_do($cmd);
		if (!defined($ret) || $ret) { last; }
	}
	@{$CMDSref} = ();
	return $ret;
}

sub ar_do {
	my ($cmd, $line, $ret);
	my $exit_code = 0;
	($cmd) = (@_);
	my $cmdText = $cmd;
	$cmdText =~ s/"//g; 
	VCSAG_LOG_MSG("N", "Issuing command: $cmdText", 60010, "$cmdText");
	my $pid = open(INPUT,"$cmd 2>&1 |");
	if (!defined($pid)) {
		#
		# open failed
		#
		my $err = "$!";
		VCSAG_LOG_MSG("E", "Command failed with error $err",
			60014, "$err");
		return undef;
	}

	#
	# open succeeded
	#
	while ($line = <INPUT>) {
		chomp($line);
		$ret = ($ret) ? "$ret\n$line" : $line;
	}
	close INPUT;
	$exit_code = $?;
	if ($exit_code) {
		my $status = $exit_code / 256;
		VCSAG_LOG_MSG("E",
			"Command failed with error: $ret and exit code: $status",
			60013, "$ret", "$status");

	} else {
		VCSAG_LOG_MSG("N", "Command succeeded", 60011);
	}
	return $exit_code;
}

sub create_lockfile {
    if ($OS ne "Windows NT") {
        my $self = shift;
        return $self->ArrayRep::create_lockfile();
    }
    my $self = shift;
    my $vcshome = $vcs_home;
    $vcshome =~ s/\"//g;
	my $LockFile = $vcshome . "/bin/SRDFSnap/.VCS_SRDFSnap_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
	open(lfh, ">$LockFile");
	close(lfh);
}

sub remove_lockfile {
    if ($OS ne "Windows NT") {
        my $self = shift;
        return $self->ArrayRep::remove_lockfile();
    }

    my $self = shift;
    my $vcshome = $vcs_home;
    $vcshome =~ s/\"//g;
    my $LockFile = $vcshome . "/bin/SRDFSnap/.VCS_SRDFSnap_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
    unlink("$LockFile");
}

sub lockfile_exists {
    if ($OS ne "Windows NT") {
        my $self = shift;
        return $self->ArrayRep::check_lockfile();
    }

    my $self = shift;
    my $vcshome = $vcs_home;
    $vcshome =~ s/\"//g;
    my $LockFile = $vcshome . "/bin/SRDFSnap/.VCS_SRDFSnap_" . $self->{'resname'} . "_Online_Lockfile.DO-NOT-REMOVE";
    if ( -f "$LockFile") {
        return 1;
    } else {
        return 0;
    }
}

#sub will call scandisk action on all slave node
#if -actionargs  $online=1 which will online disks required for firedrill and offline other disk depends upon get_tag.
#if -actionargs $online=0 then it will only offline disks based on get_tag and get_disks sub.


sub online_disk_on_slaves {
        my ($self, $nodename) = @_;
        my $online = 1;
	my $ret = "";
        if ( $osType !~ /^MSWin/i) {
		my ($case) =  $self->{'sotype'};
		my ( $searchdev, $tag, $action_arg);
		$self->get_tag ($case, \$searchdev, \$tag);
		$action_arg =  $searchdev;
		
		$action_arg =~ s/\\/\\\\/g;
		my $srdf_res_name = $self->{'target'};
		VCSAG_LOG_MSG("I","calling dev_scan action on slave node[$nodename] Resources name [$srdf_res_name] srdfsnap resource[$self->{'resname'}] actionargs[$action_arg]", 60015, "$nodename", "$srdf_res_name","$self->{'resname'}", "$action_arg");
		$ret = `$ag_i18n_inc::HARES -action $srdf_res_name  dev_scan -actionargs $action_arg -sys $nodename`;
                if($ret){
			VCSAG_LOGDBG_MSG(1, "dev_scan on node: [$nodename] return [$ret]");
                }
        }
}

sub extract_CLI_version{
        my ($self) = shift;
        my ($arrref) = @_;
        my @tmparr = @{$arrref};
        my $version = "";
        my $pattern= qr/(\d.\d.\d)/;
        foreach(@tmparr) {
                my $out1 = "";
                $out1 = $_;
                if(($out1 =~ /Symmetrix Command/) && ($out1 =~ /$pattern/)) {
                        $self->{'symcli_ver'}=$1;
                        VCSAG_LOG_MSG("I", "Symcli version is: $self->{'symcli_ver'}", 60035, "$self->{'symcli_ver'}");
                        last;
                }
        }
}

1;
